"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Generator = require("yeoman-generator");
const lodash_1 = require("lodash");
const fs = require("fs");
const path = require("path");
const colors = require("colors");
const yosay = require("yosay");
const dargs = require("dargs");
const utils_1 = require("./scripts/utils");
const install_1 = require("./scripts/install");
const prompts_1 = require("./scripts/prompts");
const configurators = require("./scripts/configs");
module.exports = class extends Generator {
    constructor(args, options) {
        super(args, options);
        this.data = {};
        this.existingProject = false;
        this.isAngularProject = false;
        this.utils = new utils_1.Utils({ yo: this });
        this.headless = process.argv.slice(2).indexOf('--headless') !== -1;
        this.skipNpmInstall = process.argv.slice(2).indexOf('--skip-npm-install') !== -1;
        this.option('package-manager', {
            description: 'preferred package manager (npm, yarn, pnpm)',
            type: String,
            alias: 'pm',
            default: 'npm'
        });
    }
    initializing() {
        this.data.sppp = require(path.join(__dirname, '../package.json'));
        const spppVersion = this.data && this.data.sppp && this.data.sppp.version || 'unknown';
        this.log(yosay(`Welcome to ${colors.yellow('SharePoint Pull-n-Push')} generator (v.${spppVersion})!`));
        this.appname = lodash_1.kebabCase(this.appname);
        this.appname = this.appname || this.config.get('appname') || 'sharepoint-app';
        this.config.set('app.name', this.appname);
        this.config.set('sppp.version', spppVersion);
        this.config.save();
        (() => {
            const packagePath = this.utils.resolveDestPath('package.json');
            if (fs.existsSync(packagePath)) {
                this.existingProject = true;
                this.packageData = require(packagePath);
            }
            const angularCliPath = this.utils.resolveDestPath('.angular-cli.json');
            if (fs.existsSync(angularCliPath)) {
                this.log(`\n${colors.yellow.bold('Angular project is detected, SPPP will be installed above safely.\n')}`);
                this.isAngularProject = true;
            }
        })();
    }
    prompting() {
        const done = this.async();
        if (this.headless) {
            return done();
        }
        (() => __awaiter(this, void 0, void 0, function* () {
            yield prompts_1.promptQuestions(this.data, this).then((answers) => {
                this.data.answers = Object.assign(Object.assign({}, this.data.answers), answers);
            });
            yield prompts_1.promptAdditionalQuestions(this.data, this).then((answers) => {
                if (answers.additional) {
                    let presets = answers.additional.presets || [];
                    if (presets.indexOf('office-ui-fabric') !== -1 && presets.indexOf('react') === -1) {
                        presets = ['react', ...presets];
                    }
                    answers.additional = Object.assign(Object.assign({}, answers.additional), { presets });
                }
                this.data.answers = Object.assign(Object.assign({}, this.data.answers), answers);
            });
            done();
        }))()
            .catch((error) => this.log(`\n${colors.red.bold(error.message)}\n`));
    }
    configuring() {
        if (!this.headless) {
            this.config.set('app.name', this.data.answers && this.data.answers.name);
            this.config.set('app.description', this.data.answers && this.data.answers.description);
            this.config.set('app.author', this.data.answers && this.data.answers.author);
            this.config.set('conf.spFolder', this.data.answers && this.data.answers.spFolder);
            this.config.set('conf.distFolder', this.data.answers && this.data.answers.distFolder);
            const additional = this.data.answers && this.data.answers.additional ? this.data.answers.additional : [];
            Object.keys(additional).forEach((key) => this.config.set(`conf.additional.${key}`, additional[key]));
            this.config.save();
        }
        else {
            const config = this.config.getAll();
            this.data.answers = {
                name: config['app.name'],
                description: config['app.description'],
                author: config['app.author'],
                spFolder: config['conf.spFolder'],
                distFolder: config['conf.distFolder'],
                additional: Object.keys(config)
                    .filter((key) => key.indexOf('conf.additional.') !== -1)
                    .reduce((res, key) => {
                    res[key.replace('conf.additional.', '')] = config[key];
                    return res;
                }, {})
            };
        }
    }
    writing() {
        this.log(`\n${colors.yellow.bold('Writing files')}`);
        if (!this.existingProject) {
            this.utils.writeJsonSync('package.json', configurators.packageJson(this.data));
        }
        const appJson = configurators.configAppJson(this.data);
        this.utils.writeJsonSync('config/app.json', appJson);
        this.utils.writeJsonSync('tsconfig.json', configurators.tsconfigJson(this.data));
        this.utils.writeJsonSync('tslint.json', configurators.tslintJson(this.data));
        this.utils.copyFile('gulpfile.js', null, true);
        this.utils.copyFile('gitignore', '.gitignore');
        this.utils.copyFile('env', '.env');
        this.utils.copyFile('webpack.config.js');
        if (this.data.answers && this.data.answers.additional && this.data.answers.additional.customTasks) {
            this.utils.copyFile('tools/tasks/example.js');
            this.utils.copyFile('tools/tasks/customDataLoader.js');
        }
        if (!this.isAngularProject) {
            this.utils.createFolder('src/scripts');
            this.utils.createFolder('src/libs');
            this.utils.createFolder('src/styles');
            this.utils.createFolder('src/fonts');
            this.utils.createFolder('src/images');
            this.utils.createFolder('src/masterpage/layouts');
            this.utils.createFolder('src/webparts');
            this.utils.createFolder('dist');
            this.utils.copyFolder('src', 'src');
            if (this.data.answers && this.data.answers.additional) {
                const presets = this.data.answers.additional.presets || [];
                if (presets.indexOf('react') !== -1) {
                    this.utils.copyFolder('presets/react', 'src');
                }
                if (presets.indexOf('office-ui-fabric') !== -1) {
                    this.utils.copyFolder('presets/react', 'src');
                    this.utils.copyFolder('presets/office-ui-fabric', 'src');
                }
                const confPresets = this.data.answers.additional.confPresets || [];
                if (confPresets.indexOf('eslint') !== -1) {
                    this.utils.writeJsonSync('.eslintrc', configurators.eslintJson(this.data));
                }
                if (confPresets.indexOf('prettier') !== -1) {
                    this.utils.writeJsonSync('.prettierrc', configurators.prettierJson(this.data));
                }
                if (confPresets.indexOf('editorconfig') !== -1) {
                    this.utils.copyFile('editorconfig', '.editorconfig');
                }
            }
        }
        this.utils.copyFolder('vscode', '.vscode');
        if (this.data.answers && this.data.answers.additional && this.data.answers.additional.sslCerts) {
            this.utils.copyFolder('config/ssl', 'config/ssl');
        }
        this.log(`${colors.green('Done writing')}`);
    }
    install() {
        this.log(`\n${colors.yellow.bold('Installing dependencies')}\n`);
        const done = this.async();
        if (this.isAngularProject) {
            install_1.npmDependencies.devDependencies ? install_1.npmDependencies.devDependencies.push('concurrently') : install_1.npmDependencies.devDependencies = ['concurrently'];
            this.packageData.scripts.spdev = 'concurrently --kill-others \"ng build --watch\" \"gulp watch\"';
            this.utils.writeJsonSync('package.json', this.packageData, true);
        }
        (() => __awaiter(this, void 0, void 0, function* () {
            let next = true;
            let installer = null;
            let depOptions = null;
            let devDepOptions = null;
            depOptions = { 'save': true };
            if (this.options['package-manager'] === 'pnpm') {
                next && (yield this.utils.execPromise('pnpm --version').then(() => {
                    installer = (dep, opt) => {
                        opt = Object.assign(Object.assign({}, opt), { 'shamefully-hoist': true });
                        const args = ['add'].concat(dep).concat(dargs(opt));
                        this.spawnCommandSync('pnpm', args);
                    };
                    devDepOptions = { 'save-dev': true };
                    next = false;
                }).catch(() => next = true));
            }
            if (this.options['package-manager'] === 'yarn') {
                next && (yield this.utils.execPromise('yarn --version').then(() => {
                    installer = this.yarnInstall.bind(this);
                    devDepOptions = { 'dev': true };
                    next = false;
                }).catch(() => next = true));
            }
            next && (() => {
                installer = this.npmInstall.bind(this);
                devDepOptions = { 'save-dev': true };
            })();
            let dependencies = install_1.npmDependencies.dependencies || [];
            let devDependencies = install_1.npmDependencies.devDependencies || [];
            const presets = [
                ...this.data.answers && this.data.answers.additional && this.data.answers.additional.presets || [],
                ...this.data.answers && this.data.answers.additional && this.data.answers.additional.confPresets || []
            ];
            presets.forEach((preset) => {
                if (typeof install_1.presetDependencies[preset] !== 'undefined') {
                    const { dependencies: dep, devDependencies: devDep } = install_1.presetDependencies[preset];
                    if (dep) {
                        dependencies = [...dependencies, ...dep];
                    }
                    if (devDep) {
                        devDependencies = [...devDependencies, ...devDep];
                    }
                }
            });
            const mapDep = (dep) => {
                return typeof dep === 'string' ? dep : dep.join('@');
            };
            const pkgJson = this.utils.readJsonSync('package.json');
            if (pkgJson) {
                const reduceDependencies = (res = {}, dep) => {
                    if (typeof dep === 'string') {
                        res[dep] = 'latest';
                    }
                    if (Array.isArray(dep) && dep.length === 2) {
                        res[dep[0]] = dep[1];
                    }
                    return res;
                };
                pkgJson.dependencies = Object.assign(Object.assign({}, (pkgJson.dependencies || {})), dependencies.reduce(reduceDependencies, {}));
                pkgJson.devDependencies = Object.assign(Object.assign({}, (pkgJson.devDependencies || {})), devDependencies.reduce(reduceDependencies, {}));
                this.utils.writeJsonSync('package.json', pkgJson, true);
            }
            if (!this.skipNpmInstall) {
                installer(dependencies.map(mapDep), depOptions);
                installer(devDependencies.map(mapDep), devDepOptions);
            }
            done();
        }))()
            .catch((error) => this.log(`\n${colors.red.bold(error.message)}\n`));
    }
    end() {
        this.log(`\n${colors.yellow.bold('Installation successful!')}`);
        this.log(`\n${colors.gray(`Run \`${colors.blue.bold('npm run config')}\` to configure SharePoint connection.`)}`);
    }
};
